<?php


	// requires: xmlrpc.inc

	
	function php_2_xmlrpc_type($phptype)
	{
		switch(strtolower($phptype))
		{
			case 'string':
				return $GLOBALS['xmlrpcString'];
			case 'integer':
			case $GLOBALS['xmlrpcInt']: // 'int'
			case $GLOBALS['xmlrpcI4']:
				return $GLOBALS['xmlrpcInt'];
			case 'double':
				return $GLOBALS['xmlrpcDouble'];
			case 'boolean':
				return $GLOBALS['xmlrpcBoolean'];
			case 'array':
				return $GLOBALS['xmlrpcArray'];
			case 'object':
				return $GLOBALS['xmlrpcStruct'];
			case $GLOBALS['xmlrpcBase64']:
			case $GLOBALS['xmlrpcStruct']:
				return strtolower($phptype);
			case 'resource':
				return '';
			default:
				if(class_exists($phptype))
				{
					return $GLOBALS['xmlrpcStruct'];
				}
				else
				{
					// unknown: might be any 'extended' xmlrpc type
					return $GLOBALS['xmlrpcValue'];
				}
		}
	}

	
	function xmlrpc_2_php_type($xmlrpctype)
	{
		switch(strtolower($xmlrpctype))
		{
			case 'base64':
			case 'datetime.iso8601':
			case 'string':
				return $GLOBALS['xmlrpcString'];
			case 'int':
			case 'i4':
				return 'integer';
			case 'struct':
			case 'array':
				return 'array';
			case 'double':
				return 'float';
			case 'undefined':
				return 'mixed';
			case 'boolean':
			case 'null':
			default:
				// unknown: might be any xmlrpc type
				return strtolower($xmlrpctype);
		}
	}

	
	function wrap_php_function($funcname, $newfuncname='', $extra_options=array())
	{
		$buildit = isset($extra_options['return_source']) ? !($extra_options['return_source']) : true;
		$prefix = isset($extra_options['prefix']) ? $extra_options['prefix'] : 'xmlrpc';
		$encode_php_objects = isset($extra_options['encode_php_objs']) ? (bool)$extra_options['encode_php_objs'] : false;
		$decode_php_objects = isset($extra_options['decode_php_objs']) ? (bool)$extra_options['decode_php_objs'] : false;
		$catch_warnings = isset($extra_options['suppress_warnings']) && $extra_options['suppress_warnings'] ? '@' : '';

		if(version_compare(phpversion(), '5.0.3') == -1)
		{
			// up to php 5.0.3 some useful reflection methods were missing
			error_log('XML-RPC: cannot not wrap php functions unless running php version bigger than 5.0.3');
			return false;
		}
		if((is_array($funcname) && !method_exists($funcname[0], $funcname[1])) || !function_exists($funcname))
		{
			error_log('XML-RPC: function to be wrapped is not defined: '.$funcname);
			return false;
		}
		else
		{
			// determine name of new php function
			if($newfuncname == '')
			{
				if(is_array($funcname))
				{
					$xmlrpcfuncname = "{$prefix}_".implode('_', $funcname);
				}
				else
				{
					$xmlrpcfuncname = "{$prefix}_$funcname";
				}
			}
			else
			{
				$xmlrpcfuncname = $newfuncname;
			}
			while($buildit && function_exists($xmlrpcfuncname))
			{
				$xmlrpcfuncname .= 'x';
			}

			// start to introspect PHP code
			$func =& new ReflectionFunction($funcname);
			if($func->isInternal())
			{
				// Note: from PHP 5.1.0 onward, we will possibly be able to use invokeargs
				// instead of getparameters to fully reflect internal php functions ?
				error_log('XML-RPC: function to be wrapped is internal: '.$funcname);
				return false;
			}

			// retrieve parameter names, types and description from javadoc comments

			// function description
			$desc = '';
			// type of return val: by default 'any'
			$returns = $GLOBALS['xmlrpcValue'];
			// desc of return val
			$returnsDocs = '';
			// type + name of function parameters
			$paramDocs = array();

			$docs = $func->getDocComment();
			if($docs != '')
			{
				$docs = explode("\n", $docs);
				$i = 0;
				foreach($docs as $doc)
				{
					$doc = trim($doc, " \r\t
	function wrap_xmlrpc_method($client, $methodname, $extra_options=0, $timeout=0, $protocol='', $newfuncname='')
	{
		// mind numbing: let caller use sane calling convention (as per javadoc, 3 params),
		// OR the 2.0 calling convention (no ptions) - we really love backward compat, don't we?
		if (!is_array($extra_options))
		{
			$signum = $extra_options;
			$extra_options = array();
		}
		else
		{
			$signum = isset($extra_options['signum']) ? (int)$extra_options['signum'] : 0;
			$timeout = isset($extra_options['timeout']) ? (int)$extra_options['timeout'] : 0;
			$protocol = isset($extra_options['protocol']) ? $extra_options['protocol'] : '';
			$newfuncname = isset($extra_options['new_function_name']) ? $extra_options['new_function_name'] : '';
		}
		//$encode_php_objects = in_array('encode_php_objects', $extra_options);
		//$verbatim_client_copy = in_array('simple_client_copy', $extra_options) ? 1 :
		//	in_array('build_class_code', $extra_options) ? 2 : 0;

		$encode_php_objects = isset($extra_options['encode_php_objs']) ? (bool)$extra_options['encode_php_objs'] : false;
		$decode_php_objects = isset($extra_options['decode_php_objs']) ? (bool)$extra_options['decode_php_objs'] : false;
		$simple_client_copy = isset($extra_options['simple_client_copy']) ? (int)($extra_options['simple_client_copy']) : 0;
		$buildit = isset($extra_options['return_source']) ? !($extra_options['return_source']) : true;
		$prefix = isset($extra_options['prefix']) ? $extra_options['prefix'] : 'xmlrpc';
		if (isset($extra_options['return_on_fault']))
		{
			$decode_fault = true;
			$fault_response = $extra_options['return_on_fault'];
		}
		else
		{
			$decode_fault = false;
			$fault_response = '';
		}
		$debug = isset($extra_options['debug']) ? ($extra_options['debug']) : 0;

		$msgclass = $prefix.'msg';
		$valclass = $prefix.'val';
		$decodefunc = 'php_'.$prefix.'_decode';

		$msg =& new $msgclass('system.methodSignature');
		$msg->addparam(new $valclass($methodname));
		$client->setDebug($debug);
		$response =& $client->send($msg, $timeout, $protocol);
		if($response->faultCode())
		{
			error_log('XML-RPC: could not retrieve method signature from remote server for method '.$methodname);
			return false;
		}
		else
		{
			$msig = $response->value();
			if ($client->return_type != 'phpvals')
			{
				$msig = $decodefunc($msig);
			}
			if(!is_array($msig) || count($msig) <= $signum)
			{
				error_log('XML-RPC: could not retrieve method signature nr.'.$signum.' from remote server for method '.$methodname);
				return false;
			}
			else
			{
				// pick a suitable name for the new function, avoiding collisions
				if($newfuncname != '')
				{
					$xmlrpcfuncname = $newfuncname;
				}
				else
				{
					// take care to insure that methodname is translated to valid
					// php function name
					$xmlrpcfuncname = $prefix.'_'.preg_replace(array('/\./', '/[^a-zA-Z0-9_\x7f-\xff]/'),
						array('_', ''), $methodname);
				}
				while($buildit && function_exists($xmlrpcfuncname))
				{
					$xmlrpcfuncname .= 'x';
				}

				$msig = $msig[$signum];
				$mdesc = '';
				// if in 'offline' mode, get method description too.
				// in online mode, favour speed of operation
				if(!$buildit)
				{
					$msg =& new $msgclass('system.methodHelp');
					$msg->addparam(new $valclass($methodname));
					$response =& $client->send($msg, $timeout, $protocol);
					if (!$response->faultCode())
					{
						$mdesc = $response->value();
						if ($client->return_type != 'phpvals')
						{
							$mdesc = $mdesc->scalarval();
						}
					}
				}

				$results = build_remote_method_wrapper_code($client, $methodname,
					$xmlrpcfuncname, $msig, $mdesc, $timeout, $protocol, $simple_client_copy,
					$prefix, $decode_php_objects, $encode_php_objects, $decode_fault,
					$fault_response);

				//print_r($code);
				if ($buildit)
				{
					$allOK = 0;
					eval($results['source'].'$allOK=1;');
					// alternative
					//$xmlrpcfuncname = create_function('$m', $innercode);
					if($allOK)
					{
						return $xmlrpcfuncname;
					}
					else
					{
						error_log('XML-RPC: could not create function '.$xmlrpcfuncname.' to wrap remote method '.$methodname);
						return false;
					}
				}
				else
				{
					$results['function'] = $xmlrpcfuncname;
					return $results;
				}
			}
		}
	}

	
	function wrap_xmlrpc_server($client, $extra_options=array())
	{
		$methodfilter = isset($extra_options['method_filter']) ? $extra_options['method_filter'] : '';
		$signum = isset($extra_options['signum']) ? (int)$extra_options['signum'] : 0;
		$timeout = isset($extra_options['timeout']) ? (int)$extra_options['timeout'] : 0;
		$protocol = isset($extra_options['protocol']) ? $extra_options['protocol'] : '';
		$newclassname = isset($extra_options['new_class_name']) ? $extra_options['new_class_name'] : '';
		$encode_php_objects = isset($extra_options['encode_php_objs']) ? (bool)$extra_options['encode_php_objs'] : false;
		$decode_php_objects = isset($extra_options['decode_php_objs']) ? (bool)$extra_options['decode_php_objs'] : false;
		$verbatim_client_copy = isset($extra_options['simple_client_copy']) ? !($extra_options['simple_client_copy']) : true;
		$buildit = isset($extra_options['return_source']) ? !($extra_options['return_source']) : true;
		$prefix = isset($extra_options['prefix']) ? $extra_options['prefix'] : 'xmlrpc';

		$msgclass = $prefix.'msg';
		//$valclass = $prefix.'val';
		$decodefunc = 'php_'.$prefix.'_decode';

		$msg =& new $msgclass('system.listMethods');
		$response =& $client->send($msg, $timeout, $protocol);
		if($response->faultCode())
		{
			error_log('XML-RPC: could not retrieve method list from remote server');
			return false;
		}
		else
		{
			$mlist = $response->value();
			if ($client->return_type != 'phpvals')
			{
				$mlist = $decodefunc($mlist);
			}
			if(!is_array($mlist) || !count($mlist))
			{
				error_log('XML-RPC: could not retrieve meaningful method list from remote server');
				return false;
			}
			else
			{
				// pick a suitable name for the new function, avoiding collisions
				if($newclassname != '')
				{
					$xmlrpcclassname = $newclassname;
				}
				else
				{
					$xmlrpcclassname = $prefix.'_'.preg_replace(array('/\./', '/[^a-zA-Z0-9_\x7f-\xff]/'),
						array('_', ''), $client->server).'_client';
				}
				while($buildit && class_exists($xmlrpcclassname))
				{
					$xmlrpcclassname .= 'x';
				}

				/// @todo add function setdebug() to new class, to enable/disable debugging
				$source = "class $xmlrpcclassname\n{\nvar \$client;\n\n";
				$source .= "function $xmlrpcclassname()\n{\n";
				$source .= build_client_wrapper_code($client, $verbatim_client_copy, $prefix);
				$source .= "\$this->client =& \$client;\n}\n\n";
				$opts = array('simple_client_copy' => 2, 'return_source' => true,
					'timeout' => $timeout, 'protocol' => $protocol,
					'encode_php_objs' => $encode_php_objects, 'prefix' => $prefix,
					'decode_php_objs' => $decode_php_objects
					);
				/// @todo build javadoc for class definition, too
				foreach($mlist as $mname)
				{
					if ($methodfilter == '' || preg_match($methodfilter, $mname))
					{
						$opts['new_function_name'] = preg_replace(array('/\./', '/[^a-zA-Z0-9_\x7f-\xff]/'),
							array('_', ''), $mname);
						$methodwrap = wrap_xmlrpc_method($client, $mname, $opts);
						if ($methodwrap)
						{
							if (!$buildit)
							{
								$source .= $methodwrap['docstring'];
							}
							$source .= $methodwrap['source']."\n";
						}
						else
						{
							error_log('XML-RPC: will not create class method to wrap remote method '.$mname);
						}
					}
				}
				$source .= "}\n";
				if ($buildit)
				{
					$allOK = 0;
					eval($source.'$allOK=1;');
					// alternative
					//$xmlrpcfuncname = create_function('$m', $innercode);
					if($allOK)
					{
						return $xmlrpcclassname;
					}
					else
					{
						error_log('XML-RPC: could not create class '.$xmlrpcclassname.' to wrap remote server '.$client->server);
						return false;
					}
				}
				else
				{
					return array('class' => $xmlrpcclassname, 'code' => $source, 'docstring' => '');
				}
			}
		}
	}

	
	function build_remote_method_wrapper_code($client, $methodname, $xmlrpcfuncname,
		$msig, $mdesc='', $timeout=0, $protocol='', $client_copy_mode=0, $prefix='xmlrpc',
		$decode_php_objects=false, $encode_php_objects=false, $decode_fault=false,
		$fault_response='')
	{
		$code = "function $xmlrpcfuncname (";
		if ($client_copy_mode < 2)
		{
			// client copy mode 0 or 1 == partial / full client copy in emitted code
			$innercode = build_client_wrapper_code($client, $client_copy_mode, $prefix);
			$innercode .= "\$client->setDebug(\$debug);\n";
			$this_ = '';
		}
		else
		{
			// client copy mode 2 == no client copy in emitted code
			$innercode = '';
			$this_ = 'this->';
		}
		$innercode .= "\$msg =& new {$prefix}msg('$methodname');\n";

		if ($mdesc != '')
		{
			// take care that PHP comment is not terminated unwillingly by method description
			$mdesc = "', '* /', $mdesc)."\n";
		}
		else
		{
			$mdesc = "\n";

		$innercode .= "\$res =& \${$this_}client->send(\$msg, $timeout, '$protocol');\n";
		if ($decode_fault)
		{
			if (is_string($fault_response) && ((strpos($fault_response, '%faultCode%') !== false) || (strpos($fault_response, '%faultString%') !== false)))
			{
				$respcode = "str_replace(array('%faultCode%', '%faultString%'), array(\$res->faultCode(), \$res->faultString()), '".str_replace("'", "''", $fault_response)."')";
			}
			else
			{
				$respcode = var_export($fault_response, true);
			}
		}
		else
		{
			$respcode = '$res';
		}
		if ($decode_php_objects)
		{
			$innercode .= "if (\$res->faultcode()) return $respcode; else return php_{$prefix}_decode(\$res->value(), array('decode_php_objs'));";
		}
		else
		{
			$innercode .= "if (\$res->faultcode()) return $respcode; else return php_{$prefix}_decode(\$res->value());";
		}

		$code = $code . $plist. ") {\n" . $innercode . "\n}\n";

		return array('source' => $code, 'docstring' => $mdesc);
	}

	
	function build_client_wrapper_code($client, $verbatim_client_copy, $prefix='xmlrpc')
	{
		$code = "\$client =& new {$prefix}_client('".str_replace("'", "\'", $client->path).
			"', '" . str_replace("'", "\'", $client->server) . "', $client->port);\n";

		// copy all client fields to the client that will be generated runtime
		// (this provides for future expansion or subclassing of client obj)
		if ($verbatim_client_copy)
		{
			foreach($client as $fld => $val)
			{
				if($fld != 'debug' && $fld != 'return_type')
				{
					$val = var_export($val, true);
					$code .= "\$client->$fld = $val;\n";
				}
			}
		}
		// only make sure that client always returns the correct data type
		$code .= "\$client->return_type = '{$prefix}vals';\n";
		//$code .= "\$client->setDebug(\$debug);\n";
		return $code;
	}
?>